//--------------------------------------------------------------------------------------
// Default shader by TamaFutoshi
//--------------------------------------------------------------------------------------

#include "TFShaderLibrary.hlsli"

//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VS(VS_INPUT input)
{
    PS_INPUT output = (PS_INPUT)0;
    output.Pos = float4(input.Pos, 1.0f);

    output.Pos = mul(output.Pos, mMVP);
    output.Normal = float4(input.Normal, 0.0f);
    output.Tex = input.Tex;

    output.WorldPos = float4(input.Pos, 1.0f);
    output.WorldNormal = float4(input.Normal, 0.0f);

    output.WorldPos = mul(output.WorldPos, mModel);
    output.WorldNormal = mul(output.WorldNormal, mModel);

    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PS(PS_INPUT input) : SV_TARGET
{
    //Ignore this pixel if base alpha is too small
    clip(baseColor.a > 0.01f ? 1 : -1);

    //Get color from texture
    float4 customColor = baseColor;
    float4 diffuseColor = baseColor;
    if (textureBind0) {
        diffuseColor = g_texture0.Sample(g_samWarp, input.Tex);
        customColor = diffuseColor;
    }

    //Ignore this pixel if texture alpha is too small
    clip(diffuseColor.a > 0.01f ? 1 : -1);

    //Specular strength
    float4 lightVec = normalize(lightPos - input.WorldPos);
    float4 viewVec = normalize(cameraPos - input.WorldPos);

    float NdotLightView = max(0, dot(input.Normal, normalize(lightVec + viewVec)));
    float3 specular = 0.0f;

    if (specularStrength > 0.0f) {
        float specularWeight = pow(NdotLightView, specularStrength);
        specular = saturate(specularColor * specularWeight);
    }

    //Calculate unshadowed amount
    float4 unshadowAmount = {1.0f, 1.0f, 1.0f, 1.0f};
    if (receiveShadow) {
        unshadowAmount = CalculateUnshadowedAmount(input.WorldPos);
    }

    //Light strength
	float lightStrength = dot(lightVec, normalize(input.WorldNormal)) * 0.5f + 0.5f;
    float shadowAmmount = lerp(0.1f, 0.0f, unshadowAmount.x);
    lightStrength -= shadowAmmount;

	customColor.rgb *= lightStrength;
    customColor.rgb += specular;

    if (emissionStrength && !IsBlack(emissionColor)) {
        customColor.rgb += emissionColor * emissionStrength;
    }

    if (emissionStrength && IsBlack(emissionColor)) {
        customColor.rgb += customColor.rgb * emissionStrength;
    }

    customColor.rgb = saturate(customColor.rgb);//max 1.0f

    return customColor;
}