//--------------------------------------------------------------------------------------
// TF MMD default shader by TamaFutoshi
// This file shows resources which are provided by the app. Let's start making your own effect!
// Note: This default shader has been compiled into the app. Changing this file won't effect application's default shader.
//--------------------------------------------------------------------------------------

#include "TFShaderLibrary.hlsli"

//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VS(VS_INPUT input)
{
    PS_INPUT output = (PS_INPUT)0;
    output.Pos = float4(input.Pos, 1.0f);

    output.Pos = mul(output.Pos, mMVP);
    output.Normal = float4(input.Normal, 0.0f);
    output.Tex = input.Tex;

    output.WorldPos = float4(input.Pos, 1.0f);
    output.WorldNormal = float4(input.Normal, 0.0f);

    output.WorldPos = mul(output.WorldPos, mModel);
    output.WorldNormal = mul(output.WorldNormal, mModel);

    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PS(PS_INPUT input) : SV_TARGET
{
    //Ignore this pixel if base alpha is too small
    clip(baseColor.a > 0.01f ? 1 : -1);

    //Get color from texture
    float4 customColor = baseColor;
    float4 diffuseColor = baseColor;
    if (textureBind0) {
        diffuseColor = g_texture0.Sample(g_samWarp, input.Tex);
        customColor = diffuseColor;
    }

    //Ignore this pixel if texture alpha is too small
    clip(diffuseColor.a > 0.01f ? 1 : -1);

    //Normal view
    float4 viewNormal = mul(input.Normal, mModel);
    viewNormal = mul(viewNormal, mView);
    viewNormal.xy = viewNormal.xy * 0.5f + 0.5f;
    viewNormal.xy = float2(viewNormal.x, 1.0f - viewNormal.y);//Change uv coordinate from glsl

    //Attenuation
    float3 customLightColor = lightColor.rgb;

    // TODO: Enable attenuation if needed
    // float lightDistance = distance(lightPos, input.WorldPos);
    // float atten = 1.0f / (1.0f + 0.1f * lightDistance + 0.01f * lightDistance * lightDistance);
    // atten *= 1.2f; //for ligther a bit
    // customLightColor *= atten;

    //Sphere and subtexture mapping
    if (sphereMode == SphereModeMul && textureBind1) {
        customColor.rgb *= g_texture1.Sample(g_samWarp, viewNormal.xy).rgb;
    }
    if (sphereMode == SphereModeAdd && textureBind1) {
        customColor.rgb += g_texture1.Sample(g_samWarp, viewNormal.xy).rgb;
    }
    if (sphereMode == SphereModeSubTexture && textureBind1) {
        //TODO: implement
    }

    //Change ambient a little bit
    customColor.rgb *= saturate(ambientColor + customLightColor * baseColor.rgb);

    //Specular mapping
    float4 lightVec = normalize(lightPos - input.WorldPos);
    float4 viewVec = normalize(cameraPos - input.WorldPos);

    float NdotLightView = max(0, dot(input.Normal, normalize(lightVec + viewVec)));
    float specularWeight = pow(NdotLightView, specularStrength);
    float3 specular = saturate(specularColor * customLightColor * specularWeight);

    //Calculate unshadowed amount
    float4 unshadowAmount = {1.0f, 1.0f, 1.0f, 1.0f};
    if (receiveShadow) {
        unshadowAmount = CalculateUnshadowedAmount(input.WorldPos);
    }

    //Toon mapping
	float lightStrength = dot(lightVec, normalize(input.WorldNormal)) * 0.5f + 0.5f;
    lightStrength *= unshadowAmount.x;
    float4 toon = {1.0f, 1.0f, 1.0f, 1.0f}; //White
    if (textureBind2) {
        toon = g_texture2.Sample(g_samClamp, float2(specularStrength, 1.0f - lightStrength)); //Change uv coordinate from glsl
    }

    customColor.rgb += specular;
    customColor.rgb *= toon.rgb;

    return customColor;
}